import os
import math
import apt
import psutil
import socket

def get_dmidata():
    p = os.popen("dmidecode")
    data = p.read()
    r = p.close()
    if r:
        return {}
    
    items = data.split("\n\n")
    items = items[1:]

    def parse_items(i):
        it = i.split("\n")
        it = it[2:]
        voices = {}
        for x in it:
            s = x.strip().split(":")
            if len(s) > 1:
                k = s[0].strip()
                v = s[1].strip()
            voices[k] = v
        return voices
    
    data = {"system": parse_items(items[1]),
            "board": parse_items(items[2]),
            "chassis": parse_items(items[3])}

    return data

dmi_interesting = [ ('chassis','Type',"chassis_type"),
                    ('system','Manufacturer', "system_vendor"),
                    ('system','Serial Number', "serial_serial"),
                    ('system','Product Name', "system_product")
                    ]

def human_size(number):
    suffixes = ['B', 'KB', 'MB', 'GB', 'TB', 'PB']
    human = number
    scale = 0
    if number != 0:
        scale = int((math.log10(number)) / 3)
        scale = min(scale, len(suffixes) - 1)
        human = number / (1024.0 ** scale)
    f = ('%.2f' % human).rstrip('0').rstrip('.')
    return "{} {}".format(f, suffixes[scale])
    
def get_data():
    client_data = {}

    dmidata = get_dmidata()    
    # collect data from DMI
    for k in dmi_interesting:
        if k[0] in dmidata.keys():
            if k[1] in dmidata[k[0]].keys():
                client_data[k[2]] = dmidata[k[0]][k[1]]

    # collect video size
    try:
        video_size = open('/var/tmp/fuss-display-size').read().strip()
        client_data['video_size'] = video_size
    except:
        pass

    # network interfaces
    net_iface_data = psutils.net_if_addrs()
    for iface in net_iface_data:
        if iface == "lo":
            continue
        iface_conf = net_iface_data[iface]
        for conf in iface_conf:
            if conf.family != socket.AF_INET:
                continue
            client_data['iface {}'.format(iface)] = conf.address

    # N. of installed packages
    p = os.popen("dpkg -l | wc -l", "r")
    client_data['ninstalledpackages'] = int(p.read()) - 5

    # fuss-{client,server) version
    pkg_cache = apt.Cache()
    checked_packages = ['fuss-client', 'fuss-server']
    for package in checked_packages:
        if pkg_cache.has_key(package):
            found_package = pkg_cache[package]
            if found_package.is_installed:
                client_data[package] = found_package.versions[0].version

                
    # total amount of ram memory
    client_data['totalram'] = human_size(psutil.virtual_memory().total)
    
    # disk usage for every physical filesystems
    for part in psutil.disk_partitions():
        device = part.device
        mpoint = part.mountpoint
        usage = psutil.disk_usage(mpoint)
        usage_percent = usage.percent
        total_size = human_size(usage.total)
        client_data["{} usage".format(mpoint)] = "{}%".format(usage_percent)
        client_data["{} size".format(device)] = total_size

    # video card
    p = os.popen("lspci | grep VGA",'r')
    data = p.read().strip()
    client_data['videocard'] = data[35:]
    
    # cpu type
    p = os.popen("grep 'model name' /proc/cpuinfo | uniq | cut -d ':' -f 2",'r')
    data = p.read().strip()
    client_data['cputype'] = data
    
    # cpu speed
    p = os.popen("grep 'cpu MHz' /proc/cpuinfo | head -1 | cut -d ':' -f 2",'r')
    data = p.read().strip()
    client_data['cpuspeed'] = data

    return client_data
